#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
using System.Windows.Forms;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Indentifies Price action patterns in mult instruments, and alerts.
    /// </summary>
    [Description("Indentifies Price action patterns in mult instruments, and alerts.")]
    public class MultiInstrumentPriceActionIndicator : Strategy
    {
        #region Variables
		
		private int stopExtraPips = 5;
		private int entryExtraPips = 3;
		private int alertCount = 0;
		private DateTime lastBarTime;
		private Font formFont = new Font("Courier", 10, GraphicsUnit.Point);
		private Form form = new Form();
		private string currentTimeframe;
		private DataGridView alertsGrid = new DataGridView();
		private int minPinBarBodyNoseRatio =1;
		private int multiplerSize = 0;
		
		
        #endregion

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose = true;
			
			AddInstruments();
			
			BuildAlertsForm();
			
        }
		
		private void BuildAlertsForm()
		{
			alertsGrid.Columns.Add("Action","PA");
			alertsGrid.Columns.Add("Signal","Signal");
			alertsGrid.Columns.Add("Pair","Pair");
			alertsGrid.Columns.Add("TimeFrame","T/F");
			alertsGrid.Columns.Add("Date","Date");
			alertsGrid.Columns.Add("Time","Time");
			alertsGrid.Columns.Add("LongEntry","Long Entry");
			alertsGrid.Columns.Add("LongStop","Long Stop");
			alertsGrid.Columns.Add("ShortEntry","Short Entry");
			alertsGrid.Columns.Add("ShortStop","Short Stop");
			alertsGrid.Columns.Add("Stop Pips","Stop Pips");
			alertsGrid.Font = formFont;
			alertsGrid.Dock = DockStyle.Fill;
			alertsGrid.ClipboardCopyMode = DataGridViewClipboardCopyMode.EnableWithoutHeaderText;
			alertsGrid.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
			form.Closing += ClosingForm;
			form.Name = "Alerts Form";
			form.Controls.Add(alertsGrid);
			form.Text = string.Format("Price Action Alerts");
			form.Width = 1000;
			form.Font = formFont;
		

		}
		
		#region Add Instruments and Time Frames
		
		private void AddInstruments()
		{
		/*	#region 30 Mins
		Add("$AUDUSD", PeriodType.Minute, 30);
			Add("$EURCHF", PeriodType.Minute, 30);
			Add("$EURGBP", PeriodType.Minute, 30);
			Add("$EURJPY", PeriodType.Minute, 30);
			Add("$EURUSD", PeriodType.Minute, 30);
			Add("$GBPJPY", PeriodType.Minute, 30);
			Add("$GBPUSD", PeriodType.Minute, 30);
			Add("$USDCAD", PeriodType.Minute, 30);
			Add("$USDCHF", PeriodType.Minute, 30);
			Add("$USDJPY", PeriodType.Minute, 30);
			#endregion
		*/	
			#region Hourly
		 	Add("$AUDCAD", PeriodType.Minute, 60);
		 	Add("$AUDJPY", PeriodType.Minute, 60);
//		 	Add("$AUDNZD", PeriodType.Minute, 60);
			Add("$AUDUSD", PeriodType.Minute, 60);
		 	Add("$CHFJPY", PeriodType.Minute, 60);
		 	Add("$EURAUD", PeriodType.Minute, 60);
//		 	Add("$EURCAD", PeriodType.Minute, 60);
			Add("$EURCHF", PeriodType.Minute, 60);
			Add("$EURGBP", PeriodType.Minute, 60);
			Add("$EURJPY", PeriodType.Minute, 60);
			Add("$EURUSD", PeriodType.Minute, 60);
			Add("$GBPCHF", PeriodType.Minute, 60);
			Add("$GBPJPY", PeriodType.Minute, 60);
			Add("$GBPUSD", PeriodType.Minute, 60);
			Add("$NZDUSD", PeriodType.Minute, 60);
			Add("$USDCAD", PeriodType.Minute, 60);
			Add("$USDCHF", PeriodType.Minute, 60);
			Add("$USDJPY", PeriodType.Minute, 60);
			#endregion
			
			#region 4 Hourly
			Add("$AUDCAD", PeriodType.Minute, 240);
			Add("$AUDJPY", PeriodType.Minute, 240);
//			Add("$AUDNZD", PeriodType.Minute, 240);
			Add("$AUDUSD", PeriodType.Minute, 240);
			Add("$CHFJPY", PeriodType.Minute, 240);
			Add("$EURAUD", PeriodType.Minute, 240);
//			Add("$EURCAD", PeriodType.Minute, 240);
			Add("$EURCHF", PeriodType.Minute, 240);
			Add("$EURGBP", PeriodType.Minute, 240);
			Add("$EURJPY", PeriodType.Minute, 240);
			Add("$EURUSD", PeriodType.Minute, 240);
			Add("$GBPCHF", PeriodType.Minute, 240);
			Add("$GBPJPY", PeriodType.Minute, 240);
			Add("$GBPUSD", PeriodType.Minute, 240);
			Add("$NZDUSD", PeriodType.Minute, 240);
			Add("$USDCAD", PeriodType.Minute, 240);
			Add("$USDCHF", PeriodType.Minute, 240);
			Add("$USDJPY", PeriodType.Minute, 240);
			#endregion
			
			#region Daily
			Add("$AUDCAD", PeriodType.Day, 1);
			Add("$AUDJPY", PeriodType.Day, 1);
//			Add("$AUDNZD", PeriodType.Day, 1);
			Add("$AUDUSD", PeriodType.Day, 1);
			Add("$CHFJPY", PeriodType.Day, 1);
			Add("$EURAUD", PeriodType.Day, 1);
//			Add("$EURCAD", PeriodType.Day, 1);
			Add("$EURCHF", PeriodType.Day, 1);
			Add("$EURGBP", PeriodType.Day, 1);
			Add("$EURJPY", PeriodType.Day, 1);
			Add("$EURUSD", PeriodType.Day, 1);
			Add("$GBPCHF", PeriodType.Day, 1);
			Add("$GBPJPY", PeriodType.Day, 1);
			Add("$GBPUSD", PeriodType.Day, 1);
			Add("$NZDUSD", PeriodType.Day, 1);
			Add("$USDCAD", PeriodType.Day, 1);
			Add("$USDCHF", PeriodType.Day, 1);
			Add("$USDJPY", PeriodType.Day, 1);
			#endregion
			
			#region Weekly
			Add("$AUDCAD", PeriodType.Week, 1);
			Add("$AUDJPY", PeriodType.Week, 1);
//			Add("$AUDNZD", PeriodType.Week, 1);
			Add("$AUDUSD", PeriodType.Week, 1);
			Add("$CHFJPY", PeriodType.Week, 1);
			Add("$EURAUD", PeriodType.Week, 1);
//			Add("$EURCAD", PeriodType.Week, 1);
			Add("$EURCHF", PeriodType.Week, 1);
			Add("$EURGBP", PeriodType.Week, 1);
			Add("$EURJPY", PeriodType.Week, 1);
			Add("$EURUSD", PeriodType.Week, 1);
			Add("$GBPCHF", PeriodType.Week, 1);
			Add("$GBPJPY", PeriodType.Week, 1);
			Add("$GBPUSD", PeriodType.Week, 1);
			Add("$NZDUSD", PeriodType.Week, 1);
			Add("$USDCAD", PeriodType.Week, 1);
			Add("$USDCHF", PeriodType.Week, 1);
			Add("$USDJPY", PeriodType.Week, 1);
			#endregion
		}
		
		#endregion

		#region On Bar Update
		
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			double longEntry, longStop, shortEntry, shortStop;
			int pinBarNoseSize = 0;
			
			//Ignore the charts bars as it will be included already
			if (BarsInProgress == 0)
				return;
			
			multiplerSize = (int) (1 / TickSize);
			
			GetCurrentTimeFrame();
			
			#region Inside Bar
			
			bool isIB =  IsInsideBar(out longEntry, out longStop, out shortEntry, out shortStop);
			
			if (isIB)
			{
				Alert(Terms.PA_IB, Terms.SIG_BREAKOUT, longEntry, longStop, shortEntry, shortStop);
				return;
			}
			
			bool isBUPB =  IsBullishPinBar(out longEntry, out longStop, out pinBarNoseSize);

			if (isBUPB)
			{
				Alert(string.Format("{0} (nose size {1})",Terms.PA_PB, pinBarNoseSize), Terms.SIG_BUY, longEntry, longStop, 0, 0);
				return;
			}
			
			bool isBEPB =  IsBearishPinBar(out shortEntry, out shortStop, out pinBarNoseSize);

			if (isBEPB)
			{
				Alert(string.Format("{0} (nose size {1})",Terms.PA_PB, pinBarNoseSize), Terms.SIG_SELL, 0, 0, shortEntry, shortStop);
				return;
			}
			
			bool isBUOB =  IsBullishOutsideBar(out longEntry, out longStop);

			if (isBUOB)
			{
				Alert(Terms.PA_BUOB, Terms.SIG_BUY, longEntry, longStop, 0, 0);
				return;
			}
			
			bool isBEOB =  IsBearishOutsideBar(out shortEntry, out shortStop);

			if (isBEOB)
			{
				Alert(Terms.PA_BEOB, Terms.SIG_SELL, 0, 0, shortEntry, shortStop);
				return;
			}
			
			bool isDBLHC =  IsDoubleLowHighCloseBars(out longEntry, out longStop);

			if (isDBLHC)
			{
				Alert(Terms.PA_DBLHC, Terms.SIG_BUY, longEntry, longStop, 0, 0);
				return;
			}
			
			bool isDBHLC =  IsDoubleHighLowCloseBars(out shortEntry, out shortStop);

			if (isDBHLC)
			{
				Alert(Terms.PA_DBHLC, Terms.SIG_SELL, 0, 0, shortEntry, shortStop);
				return;
			}
			
			#endregion
        }
		
			
		private void GetCurrentTimeFrame()
		{
			switch (BarsArray[BarsInProgress].Period.Id)
			{
				case PeriodType.Week:
					currentTimeframe = Terms.TF_WEEK;
					break;
				case PeriodType.Day:
					currentTimeframe = Terms.TF_DAY;
					break;
				case PeriodType.Minute:
				{
					switch (BarsArray[BarsInProgress].Period.Value)
					{
						case 240:
							currentTimeframe = Terms.TF_4HOUR;
							break;
						case 60:
							currentTimeframe = Terms.TF_HOUR;
							break;
						case 30:
							currentTimeframe = Terms.TF_30MIN;
							break;
					}
					break;
				}
			}
		}
			
		#endregion
			
		
		#region Price Action Pattern Recognition Methods
		
		private bool IsInsideBar(out double longEntry, out double longStop, 
			out double shortEntry, out double shortStop)
		{
			longEntry = 0.00;
			longStop = 0.00;
			shortEntry = 0.00;
			shortStop = 0.00;
			
			if (High[0] < High[1] && Low[0] > Low[1])
			{
				longEntry = High[1] + (entryExtraPips * TickSize);
				longStop = Low[1] - (stopExtraPips * TickSize);
				shortEntry = Low[1] - (entryExtraPips * TickSize);
				shortStop = High[1] + (stopExtraPips * TickSize);
				
				return true;
			}
			
			return false;
		}
		
		private bool IsBullishPinBar(out double longEntry, out double longStop, out int pinBarNoseSize)
		{
			longEntry = 0.00;
			longStop = 0.00;
			pinBarNoseSize = 0;
			
			if (Close[0] > Open[0] && Low[0] < Low[1])
			{
				double bodySize = (Close[0] - Open[0]) * multiplerSize;
				double noseSize = (Open[0] - Low[0]) * multiplerSize;
				
				if ((bodySize * minPinBarBodyNoseRatio) <= noseSize)
				{
					longEntry = High[0] + (entryExtraPips * TickSize);
					longStop = Low[0] - (stopExtraPips * TickSize);
					pinBarNoseSize = (int) Math.Round(noseSize, 1);
					return true;
				}
			}
			
			return false;
		}
		
		private bool IsBearishPinBar(out double shortEntry, out double shortStop,out int pinBarNoseSize)
		{
			shortEntry = 0.00;
			shortStop = 0.00;
			
			pinBarNoseSize = 0;
			
			if (Close[0] < Open[0] && High[0] > High[1])
			{
				double bodySize = (Open[0] - Close[0]) * multiplerSize;
				double noseSize = (High[0] - Open[0]) * multiplerSize;
				
				if ((bodySize * minPinBarBodyNoseRatio) <= noseSize)
				{
					shortEntry = Low[0] - (entryExtraPips * TickSize);
					shortStop = High[0] + (stopExtraPips * TickSize);
					pinBarNoseSize = (int) Math.Round(noseSize, 1);
					return true;

				}
			}
			
			return false;
		}
		
		private bool IsBullishOutsideBar(out double longEntry, out double longStop)
		{
			longEntry = 0.00;
			longStop = 0.00;
			
			if (IsOutsideBar())
			{
				double thirdBar = (High[0] - Low[0]) / 3;
				double topThird = Low[0] + (thirdBar * 2);
				if(Close[0] > Open[0] && Close[0] >= topThird)
				{
					longEntry = High[0] + (entryExtraPips * TickSize);
					longStop = Low[0] - (stopExtraPips * TickSize);
					return true;
				}
			}
			
			return false;
		}
		
		private bool IsBearishOutsideBar(out double shortEntry, out double shortStop)
		{
			shortEntry = 0.00;
			shortStop = 0.00;
			
			if (IsOutsideBar())
			{
				double thirdBar = (High[0] - Low[0]) / 3;
				double bottomThird = Low[0] + (thirdBar);
				if(Close[0] < Open[0] && Close[0] <= bottomThird)
				{
					shortEntry = Low[0] - (entryExtraPips * TickSize);
					shortStop = High[0] + (stopExtraPips * TickSize);
				
					return true;
				}
			}
			
			return false;
		}
		
		private bool IsOutsideBar()
		{
			if (High[0] > High[1] && Low[0] < Low[1])
				return true;
			else
				return false;
		}
		
		private bool IsDoubleLowHighCloseBars(out double longEntry, out double longStop)
		{
			longEntry = 0.00;
			longStop = 0.00;
			
			if (Low[0] == Low[1] && Close[0] > High[1])
			{
				longEntry = High[0] + (entryExtraPips * TickSize);
				longStop = Low[0] - (stopExtraPips * TickSize);
				
				return true;
			}
			
			return false;
		}
		
		private bool IsDoubleHighLowCloseBars(out double shortEntry, out double shortStop)
		{
			shortEntry = 0.00;
			shortStop = 0.00;
			
			if (High[0] == High[1] && Close[0] < Low[1])
			{
				shortEntry = Low[0] - (entryExtraPips * TickSize);
				shortStop = High[0] + (stopExtraPips * TickSize);
				
				return true;
			}
			
			return false;
		}
		
		#endregion
		
		#region Helper Methods
		
		private void Alert(string pa, string signal, double longEntry, double longStop, double shortEntry, double shortStop)
		{
			Color rowBack = Color.DarkBlue;
			string instrumentName = Instrument.FullName.TrimStart('$').Insert(3,"\\");
		
            string soundFile = @"C:\Alert.wav";
			PlaySound(soundFile);
		
			int pipsStop = (int) Math.Round((longEntry - longStop) * multiplerSize, 1);;
			if (signal == Terms.SIG_SELL)
			{
				pipsStop = (int) Math.Round((shortStop - shortEntry) * multiplerSize, 1);
				rowBack = Color.DarkRed;
			}
			else if (signal == Terms.SIG_BUY)
			{
				rowBack = Color.DarkGreen;
			}
				
			object[] vals = new Object[11];
		
			vals[0] = pa;
			vals[1] = signal;
			vals[2] = instrumentName;
			vals[3] = currentTimeframe;
			vals[4] = Time[0].ToShortDateString();
			vals[5] = Time[0].ToShortTimeString();
			vals[6] = longEntry.ToString();
			vals[7] = longStop.ToString();
			vals[8] = shortEntry.ToString();
			vals[9] = shortStop.ToString();
			vals[10] = pipsStop;
			
			alertsGrid.Rows.Insert(0, vals);
			alertsGrid.Rows[0].DefaultCellStyle.ForeColor = rowBack;
			alertsGrid.Rows[0].DefaultCellStyle.Font = formFont;
			form.Show();
		}
		
		private void ClosingForm(object sender, System.ComponentModel.CancelEventArgs e)
		{
			form.Hide();
			e.Cancel = true;

		}

		#endregion
    }
	
	#region Classes
	public class Terms
	{
		#region Price Action
		public const string PA_IB = "Inside Bar";
		public const string PA_PB = "Pin Bar";
		public const string PA_BEOB = "Bearish Outside Bar";
		public const string PA_BUOB = "Bullish Outside Bar";
		public const string PA_DBLHC = "Double Bar Low Higher Close";
		public const string PA_DBHLC = "Double Bar High Lower Close";
		#endregion
		
		#region Signals
		public const string SIG_BUY = "Buy";
		public const string SIG_SELL = "Sell";
		public const string SIG_BREAKOUT = "Breakout";
		#endregion
		
		#region Time Frames
		public const string TF_WEEK = "Weekly";
		public const string TF_DAY = "Daily";
		public const string TF_4HOUR = "4 Hour";
		public const string TF_HOUR = "Hourly";
		public const string TF_30MIN = "30 Mins";
		#endregion

		#region Rating
		public const string RATE_AAA = "A++";
		public const string RATE_AA = "A+";
		public const string RATE_A = "A";
		public const string RATE_B = "B";
		public const string RATE_C = "C";
		public const string RATE_D = "D";		
		#endregion
		

		


	}
	
	#endregion
}
